/************************************************************
 *
 * COPYRIGHT (C) HITACHI,LTD. 2003 ALL RIGHTS RESERVED.
 * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 *            HITACHI CENTRAL RESEARCH LABORATORY.
 *
 * Created by M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>
 *
 ************************************************************/

#include "command.h"
#include <linux/lkst_private.h>

#define LKST_ETYPE_DEF(event_type, hooktype, _mnemonic, event_name, arg_1, arg_2, arg_3, arg_4)    \
	[event_type] = { .mnemonic = #_mnemonic, .name = event_name, \
			 .args = {arg_1, arg_2, arg_3, arg_4 }, .flag = 0 },

struct lkst_etype_description lkst_etype_desc[LKST_ETYPE_VOID+1] = {
#include <linux/lkst_etype.h>
[LKST_ETYPE_VOID] = { .mnemonic="LKST_ETYPE_VOID", .name="void event", .flag = 0}
};
#undef LKST_ETYPE_DEF

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#define BUF_SIZE 512

static char *csv_copy(const char* ptr, char *buf, char** nptr)
{
	char * ret = buf;
	int quoted;

	if (ptr == NULL) return NULL;

	while (isspace(*ptr)) ptr++; /*isspace() includes '\n'*/
	quoted = 0;
	while (*ptr!='\0') {
		if (*ptr == '\\' ) {
			ptr++;
			if ( *ptr=='\0' ) break;
		} else {
			if ( *ptr=='\"') {
				quoted = !quoted;
				ptr++;
				continue;
			}
			if ((!quoted) && *ptr==',' ) {
				ptr++;
				break;
			}
		}
		*buf++ = *ptr++;
	}
	*buf = '\0';
	if (nptr) *nptr = (char*)ptr;
	return ret;
}

static void lkst_etype_desc_free(int etype) 
{
	int i;
	if (lkst_etype_desc[etype].flag==0) return ;
	if (lkst_etype_desc[etype].mnemonic!=NULL) {
		free(lkst_etype_desc[etype].mnemonic);
		lkst_etype_desc[etype].mnemonic = NULL;
	}
	if (lkst_etype_desc[etype].name!=NULL) {
		free(lkst_etype_desc[etype].name);
		lkst_etype_desc[etype].name = NULL;
	}
	for ( i = 0; i < 4; i++)
		if (lkst_etype_desc[etype].args[i]!=NULL) {
			free(lkst_etype_desc[etype].args[i]);
			lkst_etype_desc[etype].args[i]=NULL;
		}
	lkst_etype_desc[etype].flag=0;
}


int lkst_etype_desc_load(char * fname) 
{
	FILE *fp;
	char buf[BUF_SIZE], buf2[BUF_SIZE], *ptr;
	int etype;
	int i;
	fp = fopen(fname, "r");
	if (fp==NULL) return -1;
	while (fgets(buf, BUF_SIZE, fp) != NULL ) {
		ptr = buf;
		if ( csv_copy(ptr, buf2, &ptr) == NULL ) continue;
		etype = strtol(buf2, NULL, 0);
		if ( etype <= 0 || etype >= LKST_ETYPE_MAX ) continue;
		lkst_etype_desc_free(etype);
		lkst_etype_desc[etype].flag = 1;

		if ( csv_copy(ptr, buf2, &ptr) == NULL ) continue;
		if ( buf2[0]!='\0') lkst_etype_desc[etype].mnemonic = strdup(buf2);
		if ( csv_copy(ptr, buf2, &ptr) == NULL ) continue;
		if ( buf2[0]!='\0') lkst_etype_desc[etype].name = strdup(buf2);
		for ( i = 0; i < 4; i++) {
			if ( csv_copy(ptr, buf2, &ptr) == NULL ) break;
			if ( buf2[0]!='\0')
				lkst_etype_desc[etype].args[i] = strdup(buf2);
		}
	}
	return 0;
}

void lkst_etype_desc_free_all(void) 
{ 
	int i;
	for (i=0;i<LKST_ETYPE_VOID+1;i++)
		lkst_etype_desc_free(i);
}
